/*
 * cpu.c
 *
 *  	Created on: 22.09.2012
 *      Author: 	Zippi
 *      File:		CPU Driver
 *      Beschreibung:
 *      	Inizialisiert die CPU mit bergebender Frequenz (cclock).
 *
 *      Beispiel:
 *      cpu_set_clk(60);		// Sorgt dafr, dass die CPU mit 60Mhz luft.
 */
#include <avr32/io.h>
#include "uc3_cpu.h"

#define quarz_frq	12			// Quarzfrequenz in Mhz

int iCpu_clock=0;

void cpu_set_clk(int cclock)
{
	int clockh = (cclock/(quarz_frq/2)) - 1;

	AVR32_PM.oscctrl0 = (AVR32_PM.oscctrl0 | 0x00000407); 			// Startup time  /  Oscillator Mode

	AVR32_PM.mcctrl = 			AVR32_PM_OSC0EN_MASK;				// Enable Oscillator 0 // Set Internel Clock as main oscillator

	while((AVR32_PM.poscsr & AVR32_PM_POSCSR_OSC0RDY_MASK)== 0);	// Oscillator 0 is ready


	AVR32_PM.pll[0] = 			0 << AVR32_PM_PLLOSC_OFFSET	|		// Oscillator 0 is the source for the PLL
								1 << AVR32_PM_PLLDIV_OFFSET	|		// div = 1
								clockh << AVR32_PM_PLLMUL_OFFSET |	// multi clock z.b. if clockh = 10 (10+1)*12=132/2Fvc = 66Mhz
								5 << AVR32_PM_PLLOPT_OFFSET;		// 80MHz<fvco<180MHz / fPLL = fvco / Wide Bandwidth Mode disabled


	AVR32_PM.cksel = 			0x80808080;

	AVR32_PM.pll[0] |= 			AVR32_PM_PLLEN_MASK;				// PLL enable

	while(!(AVR32_PM.poscsr & AVR32_PM_POSCSR_LOCK0_MASK));			// PLL 0 is locked, and ready to be selected as clock source

	if(cclock>30)
		AVR32_FLASHC.fcr |=			1 << AVR32_FLASHC_FWS_OFFSET;		// The flash is read with 1 wait state.

	AVR32_PM.mcctrl |= 			2 << AVR32_PM_MCSEL_OFFSET;	    	// PLL0 as main oscillator

	// ENABLE 32khz crystal
	AVR32_PM.oscctrl32 = 	1 << AVR32_PM_OSCCTRL32_MODE_OFFSET |
							1 << AVR32_PM_OSCCTRL32_OSC32EN_OFFSET;

	while((AVR32_PM.poscsr & AVR32_PM_POSCSR_OSC32RDY_MASK)== 0);	// 32khz Oscillator is ready

	iCpu_clock = cclock;
}

int cpu_get_clk()
{
	return (iCpu_clock);
}
